<?php

namespace TriNetAssess\Tests;

use GuzzleHttp\Client;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;
use GuzzleHttp\Middleware;

use TriNetAssess\OAuth2FlowAssess;
use TriNetAssess\AssessmentException;

class OAuth2FlowAssessTest extends \PHPUnit_Framework_TestCase
{
    protected $OAuth2FlowAssess;

    public function setUp() {
        $this->OAuth2FlowAssess = new OAuth2FlowAssess();
    }

    public function testSetClient() {
        $client = new Client();
        $result = $this->OAuth2FlowAssess->setClient($client);
        $this->assertInstanceOf(Client::class, $result);
     }

    public function testSetClient_invalidArgumentType() {
        $this->setExpectedException(\Exception::class);

        $client = new \stdClass();
        $result = $this->OAuth2FlowAssess->setClient($client);
    }

    public function testGetClient() {
        $client = new Client();
        $this->OAuth2FlowAssess = new OAuth2FlowAssess($client);
        $result = $this->OAuth2FlowAssess->getClient();

        $this->assertInstanceOf(Client::class, $result);
    }

    public function testGetClient_fromSetter() {
        $client = new Client();
        $this->OAuth2FlowAssess->setClient($client);

        $result = $this->OAuth2FlowAssess->getClient();
        $this->assertInstanceOf(Client::class, $result);
    }

    public function testGetClient_unsetClient() {
        $this->setExpectedException(AssessmentException::class);

        $this->OAuth2FlowAssess->getClient();
    }

    public function testSendAuthenticationRequest() {
        $client_id = 'testClientId1234';
        $client_secret = 'clientSecretShhhhhhh';

        $container = [];
        $client = $this->_setUpResponses($container, [
            new Response(200)
        ]);

        $this->OAuth2FlowAssess->setClient($client);
        $this->OAuth2FlowAssess->sendAuthenticationRequest($client_id, $client_secret);

        $this->assertCount(1, $container, 'You must send a single request in this method.');
        $authenticationRequest = $container[0]['request'];
        $requestUri = $authenticationRequest->getUri();
        $requestParams = urldecode($authenticationRequest->getBody());

        $this->assertEquals('POST', $authenticationRequest->getMethod());
        $this->assertEquals('http://example.org/oauth/token', (string) $requestUri);
        $this->assertContains('grant_type=client_credentials', $requestParams);
        $this->assertContains('client_id=testClientId1234', $requestParams);
        $this->assertContains('client_secret=clientSecretShhhhhhh', $requestParams);
    }

    public function testSendAuthorizedRequest() {
        $accessToken = 'h9w8r3902r0jr932jirjioijew0jjfej';

        $container = [];
        $client = $this->_setUpResponses($container, [
            new Response(200, [], 'You got my resources')
        ]);

        $this->OAuth2FlowAssess->setClient($client);
        $response = $this->OAuth2FlowAssess->sendAuthorizedRequest(
            new Response(200, [], json_encode([
                'access_token' => $accessToken,
                'token_type' => 'Bearer'
            ]))
        );

        $this->assertCount(1, $container, 'You must send a single request in this method.');
        $clientInfoRequest = $container[0]['request'];
        $requestUri = $clientInfoRequest->getUri();

        $this->assertTrue($clientInfoRequest->hasHeader('Authorization'));
        $this->assertEquals('Bearer ' . $accessToken, $clientInfoRequest->getHeaderLine('Authorization'));
        $this->assertEquals('http://example.org/api/some_resource', (string) $requestUri);
        $this->assertEquals('You got my resources', (string) $response->getBody());
    }

    private function _setUpResponses(&$container, $responses){
        $history = Middleware::history($container);
        $mock = new MockHandler($responses);

        $stack = HandlerStack::create($mock);
        $stack->push($history);

        $client = new Client(['handler' => $stack]);

        return $client;
    }
}